<?php

namespace Drupal\athenapdf_api;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Url;
use GuzzleHttp\Client;

/**
 * Class AthenaPdfApiConverter.
 */
class AthenaPdfConverter implements AthenaPdfConverterInterface {

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The uuid generator.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidGenerator;

  /**
   * The http client service.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Url to the endpoint.
   *
   * @var String
   */
  protected $microserviceBaseUrl;

  /**
   * Auth string of the endpoint.
   *
   * @var String
   */
  protected $microserviceAuthString;

  /**
   * Constructs a new AthenaPdfApiConverter object.
   */
  public function __construct(
    StateInterface $state,
    UuidInterface $uuidGenerator,
    Client $httpClient,
    FileSystemInterface $file_system,
    ConfigFactory $configFactory
  ) {
    $this->state = $state;
    $this->uuidGenerator = $uuidGenerator;
    $this->httpClient = $httpClient;
    $this->fileSystem = $file_system;

    $config = $configFactory->get('athenapdf_api.settings');
    $this->microserviceBaseUrl = $config->get('endpoint');
    $this->microserviceAuthString = $config->get('auth_key');
  }

  /**
   * {@inheritDoc}
   */
  public function addItem(String $markup) {
    $id = $this->uuidGenerator->generate();
    $this->state->set($this->getKey($id), $markup);
    return $id;
  }

  /**
   * {@inheritDoc}
   */
  public function burnAfterReading(String $id) {
    $key = $this->getKey($id);
    $markup = $this->state->get($key);
    if ($markup) {
      $this->state->delete($key);
    }
    return $markup;
  }

  /**
   * {@inheritDoc}
   */
  public function urlToPdf($url) {
    $targetUrl = Url::fromUri($this->microserviceBaseUrl, [
      'query' => [
        'auth' => $this->microserviceAuthString,
        'url' => $url,
      ],
    ])->toString();

    $tempName = $this->fileSystem->realpath(
      $this->fileSystem->tempnam('temporary://', 'athenapdf'));

    $this->httpClient->request('GET', $targetUrl, [
      'sink' => $tempName,
    ]);

    return $tempName;
  }

  /**
   * {@inheritDoc}
   */
  public function markupToPdf($markup) {
    $id = $this->addItem($markup);
    $markupUrl = Url::fromUri("internal:/athenapdf_api/markup/$id", [
      'absolute' => TRUE
    ])->toString();

    return $this->urlToPdf($markupUrl);
  }

  /**
   * Returns the state key of the item with given id.
   *
   * @param String $id
   *   Id of the item.
   *
   * @return string
   *   The state key associated with the given id.
   */
  protected function getKey(String $id) {
    return "athenapdf_markup_$id";
  }

  // DONE: Add a route for showing the stored html.

  // DONE: Add addHtmlToPdf method.

  // DONE: Add a config form.

  // DONE: Add a testing form with a permission.

  // TODO: Add pathToPdf method.

  // TODO: Add a README and publish.

}
